/*  disasmTCC730, decode TCC730 (CalmRISC16 / MAC2424) machine code
         to assembly mnemonics.
    file: disasmTCC730.c, main procedure

    Copyright (C) 2004  Michael R. Donat <michael@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include <sys/stat.h>
#ifndef _BORLAND_
 #include <sys/ioctl.h>
 #include <unistd.h>
 #include <sys/io.h>
#else
 #include <io.h>
#endif
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>


#include "types.h"
#include "util.h"
#include "decodeOP.h"
#include "formatOP.h"

#ifdef _BORLAND_
#define DELIM ('\\')
#else
#define DELIM ('/')
#endif
void disasmTCC730( char *file );
char *filename;

void disasmTCC730( char *file ) {
   u8 *bin;
   u32 len = loadFile( file, &bin );
   u8* end = bin + len;
   u32 ip = 0;
   while( (bin + ip) < end ) {
      u16 op1 = bu16( bin + ip );
      u16 op2 = bu16( bin + ip + 2);
      int mnnum = decodeOP( op1 );
      char* inst = formatOP( mnnum, op1, op2, ip + opsize(mnnum) );
      if( mnnum < 164 ) {
         if( opsize(mnnum) == 2 )
            printf( "%06X:( %3u) %04X     : %s\n", ip, mnnum, op1, inst );
         else if( opsize(mnnum) == 4 )
            printf( "%06X:( %3u) %04X %04X: %s\n", ip, mnnum, op1, op2, inst );
      }
      else {
         if( opsize(mnnum) == 2 )
            printf( "%06X:(E%3u) %04X     : %s\n", ip, mnnum - 164, op1, inst );
         else if( opsize(mnnum) == 4 )
            printf( "%06X:(E%3u) %04X %04X: %s\n", ip, mnnum - 164, op1, op2, inst );
      }
      ip += opsize(mnnum);
   }
}

int main( int argc, char** argv ) {
   int n = 1;
   if( argc < 2 )
      printf("Usage: %s file.o [file2.o ...]", argv[0] + strrpos( argv[0], DELIM ));
   else for(;n < argc; ++n ) {
      size_t len = strlen( argv[n] );
      int pre = strrpos( argv[n], DELIM );
      len -= pre;
      filename = (char*)malloc( len );
      strncpy( filename, argv[n] + pre + 1, len - 2 );
      strcpy( filename + len - 2, "s" );
      disasmTCC730( argv[n] );
   }
   return 0;
}


